;;;; A small library of common calendar definitions used in the CGI calendar interface stuff

;; The default calender column widht (in pixels)
(define default-column-width 180)

;; Minimum month width of a calendar
(define min-month-width 100)

;; Maximum month width of a calendar
(define max-month-width 800)

;; The directory where the calendar data (records and html files) are organized
(define data-directory "/user/normark/.public_html/cgi-bin/calendar/data/")

;; The URL of the place where the calendar cgi scripts reside.
(define calendar-url-prefix "http://www.cs.auc.dk/~normark/cgi-bin/calendar/")

;; The URL of the place where the generated calendars reside.
(define data-url-prefix (string-append calendar-url-prefix "data/"))


;; The maximum number of months in a calendar
(define max-calendar-length 24)  ; max 24 months, due to server efficiency

; (define seconds-in-a-day 86400)  from time library
(define seconds-in-a-half-day 43200)

;; The list of color. Each element is a string. Each string much correspond to
;; a symbol whose value is a rgb color list.
(define calendar-entry-color-list
  (list "light-blue" "blue" "light-green" "green" "light-red" "red" "light-purple" "purple" "light-grey" "grey" "light-yellow" "yellow"
        "light-brown" "brown" "light-orange" "orange" ))

;; Returns the rgb list of the color, as represented by the string color-string.
(define (string-to-color color-string)
  (cond ((equal? color-string "blue") (make-color 0 255 255))
        ((equal? color-string "light-blue") (make-color 185 255 255))
        ((equal? color-string "green") (make-color 15 255 158))
        ((equal? color-string "light-green") (make-color 141 255 209))
        ((equal? color-string "light-red") (make-color 249 149 149))
        ((equal? color-string "red") (make-color 255 0 0))
        ((equal? color-string "purple") (make-color 255 129 255))
        ((equal? color-string "light-purple") (make-color 255 201 255))
        ((equal? color-string "light-grey") (make-color 217 217 217))
        ((equal? color-string "grey") (make-color 178 178 178))
        ((equal? color-string "yellow") (make-color 255 255 0))
        ((equal? color-string "light-yellow") (make-color 255 255 157))
        ((equal? color-string "brown") (make-color 196 137 78))
        ((equal? color-string "light-brown") (make-color 216 177 138))
        ((equal? color-string "orange") (make-color 255 137 57))
        ((equal? color-string "light-orange") (make-color 255 185 137))
        (make-color 255 255 255)))


(define calendar-credit (con-space (con "This calendar facility is designed and programmed by Kurt Nrmark, Department of Computer Science,
                                      Aalborg University, Denmark, using " (a-tag "http://www.cs.auc.dk/~normark/laml/" "LAML") " technology.") (br)
                                      (mail-link "normark@cs.auc.dk") ", "
                                      (a-tag "http://www.cs.auc.dk/~normark")))



; ---------------------------------------------------------------------------------------------------
;;; Calendar entry selectors.

;; Return the day of a calendar entry
(define cal-entry-day (make-selector-function 1))

;; Return the month of a calendar entry
(define cal-entry-month (make-selector-function 2))

;; Return the year of a calendar entry
(define cal-entry-year (make-selector-function 3))

;; Return whether we make afternoon start of a calendar entry
(define cal-entry-afternoon? (make-selector-function 4))

;; Return the number of halfdays of a calendar entry
(define cal-entry-halfdays (make-selector-function 5))

;; Return the period lenght of a calendar entry (7 is a week).
(define cal-entry-period-length (make-selector-function 6))

;; Return the repetition of a calendar entry.
(define cal-entry-period-multiplicity (make-selector-function 7))

;; Return the text of a calendar entry.
(define cal-entry-text (make-selector-function 8))

;; Return the color of a calendar entry.
(define cal-entry-color (make-selector-function 9))

;; Return the URL of a calendar entry.
(define cal-entry-url (make-selector-function 10))

;; Return from a calendar entry whether this entry should be deleted
(define cal-entry-delete? (make-selector-function 11))

;; Calendar entry constructor 
(define (make-calendar-entry day month year afternoon? halfdays period-lenght period-multiplicity text color url delete?)
  (list day month year afternoon? halfdays period-lenght period-multiplicity text color url delete?))


; ---------------------------------------------------------------------------------------------------
; Color key selectors:

(define color-key-text-of (make-selector-function 1 "color-key-text-of"))
(define color-key-color-of (make-selector-function 2 "color-key-color-of"))

; ---------------------------------------------------------------------------------------------------


;; Read calendar info (an alist) from file-name, normally of type string.
;; If the file-name is the symbol new, always return an empty calendar info (an empty list).
;; The file-name is taken from the directory data-directory (a free variable of this function).
(define (read-calendar-info file-name)
 (if (and (symbol? file-name) (eq? file-name 'new))
     '() 
     (let ((path (string-append data-directory file-name)))
       (if (file-exists? path) (file-read path) '()))))

;; Write the calender info (an alist) on file-name.
;; The file will be written in the directory data-directory (free variable of this function).
(define (write-calendar-info info file-name)
  (let ((path (string-append data-directory file-name)))
    (save-a-list info path)))

;; Return a fonted string which tells when this page is generated.
(define (when-generated)
 (let* ((dt (date-time (current-time)))
        (date (car dt))
        (time (cadr dt)))
  (font 2 red (con "Generated: " date ", " time))))


;; Make a calendar event in the format the original calendar tool and the calendar function expects.
;; The input, entry, to this function is an entry of the form:
;; (day month year number-of-half-days text color url).
;; All elements of the list are strings.
(define (make-calendar-event entry)
 (let* ((day (as-number (cal-entry-day entry)))
        (month (as-number (cal-entry-month entry)))
        (year (as-number (cal-entry-year entry)))
        (half-days (as-number (cal-entry-halfdays entry)))
        (afternoon? (as-boolean (cal-entry-afternoon? entry)))
        (end-displacement (* (- half-days 1) seconds-in-a-half-day))
       )
  (list 
   (if afternoon? (second-count year month day 12 0 1) (second-count year month day 0 0 1))
   (+ end-displacement (if afternoon? (second-count year month day 23 59 0) (second-count year month day 11 59 59)))
   (cal-entry-text entry)
   ""
   (string-to-color (cal-entry-color entry))
   (cal-entry-url entry)
  )
 )
)


(define (news-flash)
  (if (> (string-length news-flash-string) 0)
   (con (p) 
        (table-1 1 (list 600) (list yellow) 
          (list (list 
            (con (font 4 black  news-flash-string)))))
        (p))
    ""))